

def list(survey_id, *args, **kwargs):
    tail = "survey/%s/surveyresponse" % survey_id
    params = {

    }.update(kwargs)
    return tail, params


def get(survey_id, response_id, *args, **kwargs):
    tail = "survey/%s/surveyresponse/%s" % (survey_id, response_id)
    params = {

    }.update(kwargs)
    return tail, params


def create(*args, **kwargs):
    tail = "survey/"
    params = {
        '_method': 'PUT',
    }.update(kwargs)
    return tail, params


def delete(survey_id, response_id, *args, **kwargs):
    tail = "survey/%s/surveyresponse/%s" % (survey_id, response_id)
    params = {
        '_method': 'DELETE',
    }.update(kwargs)
    return tail, params


def change(survey_id, response_id, *args, **kwargs):
    tail = "survey/%s/surveyresponse/%s" % (survey_id, response_id)
    params = {
        '_method': 'POST',
    }.update(kwargs)
    return tail, params




import oauth_helper
import requests
import hashlib
import imp


class ImproperlyConfigured(Exception):
    """ SurveyGizmo is somehow improperly configured."""
    pass


class InvalidFilter(Exception):
    """ Filter format is invalid."""
    pass


class _Config(object):
    def __init__(self, _sg, **kwargs):
        self._sg = _sg
        self._api_version = kwargs.get('api_version', 'head')
        self.auth_method = kwargs.get('auth_method', None)
        self.username = kwargs.get('username', None)
        self.password = kwargs.get('password', None)
        self.md5_hash = kwargs.get('md5_hash', None)
        self.consumer_key = kwargs.get('consumer_key', None)
        self.consumer_secret = kwargs.get('consumer_secret', None)
        self.access_token = kwargs.get('access_token', None)
        self.access_token_secret = kwargs.get('access_token_secret', None)
        self.response_type = kwargs.get('response_type', None)

    def validate(self):
        """ Perform validation check on properties.
        """
        if not self.api_version in ['v3', 'head']:
            raise ImproperlyConfigured("API version provided is invalid.")

        if not self.auth_method in ['user:pass', 'user:md5', 'oauth']:
            raise ImproperlyConfigured("No authentication method provided.")
        else:
            if self.auth_method == "user:pass":
                if not self.username or not self.password:
                    raise ImproperlyConfigured("Username and password for 'user:pass' authentication.")
            elif self.auth_method == "user:md5":
                if not self.username:
                    raise ImproperlyConfigured("Username required for 'user:md5' authentication.")
                elif not self.password and not self.md5_hash:
                    raise ImproperlyConfigured("Password or md5 hash of password required for 'user:md5' authentication.")
            elif self.auth_method == "oauth":
                if not self.consumer_key or not self.consumer_secret or \
                   not self.access_token or not self.access_token_secret:
                    raise ImproperlyConfigured("OAuth consumer key and secret, and OAuth access token and secret required for 'oauth' authentication.")

        if not self.response_type in ["json", "pson", "xml", "debug", None]:
            raise ImproperlyConfigured()

    @property
    def api_version(self):
        return self._api_version

    @api_version.setter
    def api_version(self, value):
        self._api_version = value
        self._sg.api._check_version()


class _Field(object):
    associations = {
        'surveyresponse': [
            '[question(%d)]',
            '[question(%d), option(%d)]'
            'datesubmitted',
            'istestdata',
            'status',
            'contact_id',
        ],
        'survey': [
            'createdon',
            'modifiedon',
            'title',
            'subtype',
            'team',
            'status',
        ],
        'surveycampaign': [
            '_type',
            'name',
            'ssl',
            'datecreated',
            'datemodified',
            'status',
        ],
    }


class _API(object):
    base_url = "https://restapi.surveygizmo.com/"
    head = 'v3'

    def __init__(self, _sg):
        self._sg = _sg
        self._api_version = None
        self._modules = None
        self._filters = []
        self._session = None

    def _check_version(self):
        """ update api modules, wrap callables.
        """
        if self._api_version != self._sg.config.api_version:
            self._api_version = self._sg.config.api_version
            self._modules = {}

            api_version = self.head if self._api_version == 'head' else self._api_version

            module_list = getattr(__import__('api', globals(), locals(), [api_version]), api_version).__all__
            search_path = 'surveygizmo/api/%s' % api_version

            for module_name in module_list:
                find_result = imp.find_module(module_name, [search_path])
                module = imp.load_module("_api_loaded_%s" % module_name, *find_result)

                for name, func in module.__dict__.items():
                    if callable(func) and not name.startswith('__'):
                        setattr(module, name, self._wrap(func))

                self._modules[module_name] = module

    def __getattr__(self, name):
        """ retrieve modules loaded from api
        """
        if self._modules.get(name, None) is not None:
            return self._modules[name]
        raise AttributeError(name)

    def _wrap(self, func):
        """ wrap api callable's such that their return values
            are immediately executed
        """
        def wrapper(*args, **kwargs):
            keep = kwargs.pop('keep', False)
            tail, params = func(*args, **kwargs)

            response_type = self._sg.config.response_type
            if response_type:
                tail = "%s.%s" % (tail, response_type)
            tail = "%s/%s" % (self._api_version, tail)

            return self.execute(tail, params, keep)
        return wrapper

    def add_filter(self, field, operator, value):  # , object_type=None):
        """ Add a query filter to be applied to the next API call.
            :param field: Field name to filter by
            :type field: str
            :param operator: Operator value
            :type operator: str
            :param value: Value of filter
            :type value: str
            :param object_type: Optional. Checks field for object association
            :type object_type: str

            Known Filters:
                Question                    [question(2)]                   surveyresponse
                Question Option             [question(2), option(10001)]    surveyresponse
                Date Submitted              datesubmitted                   surveyresponse
                Is Test Data                istestdata                      surveyresponse
                Status                      status                          surveyresponse
                Contact ID                  contact_id                      surveyresponse
                Creation Time               createdon                       survey
                Last Modified Time          modifiedon                      survey
                Survey Title                title                           survey
                Type of Project             subtype                         survey
                Team Survey Belongs To      team                            survey
                Status                      status                          survey
                Type of Link                type                            surveycampaign
                Name of Link                name                            surveycampaign
                Secure / Unsecure Link      ssl                             surveycampaign
                Link Created Date           datecreated                     surveycampaign
                Link Last Modified Date     datemodified                    surveycampaign
                Status                      status                          surveycampaign


            Known Operators:
                ==
                !=
                >=
                <=
                >
                <
                =           (==)
                <>          (!=)
                IS NULL     Value is True or False
                IS NOT NULL Value is True or False
                in          Value is comma separated list
        """
        i = len(self._filters)
        self._filters.append({
            'filter[field][%d]' % i: str(field),
            'filter[operator][%d]' % i: str(operator),
            'filter[value][%d]' % i: str(value),
        })

    def execute(self, tail, params, keep=False):
        """ Executes a call to the API.
            :param tail: The tail portion of the URL. This should not include
            the domain name.
            :param params: Query parameters passed to API.
            :param keep: Keep filters for next API call. Defaults to False.
        """
        self._sg.config.validate()

        for _filter in self._filters:
            params.update(_filter)
        if not keep:
            self._filters = []

        config = self._sg.config
        if config.auth_method == 'user:pass':
            params.update({
                config.auth_method: "%s:%s" % (config.username, config.password),
            })
            url = "%s%s" % (self.base_url, tail)
            response = requests.get(url, params=params)

        elif config.auth_method == 'user:md5':
            if not config.md5_hash:
                config.md5_hash = hashlib.md5(config.password).hexdigest()
            params.update({
                config.auth_method: "%s:%s" % (config.username, config.md5_hash),
            })
            url = "%s%s" % (self.base_url, tail)
            response = requests.get(url, params=params)

        else:  # 'oauth'
            if not self._session:
                self._session = oauth_helper.SGAuthService(
                    config.consumer_key, config.consumer_secret,
                    config.access_token, config.access_token_secret
                ).get_session()

            response = self._session.get(tail, params)

        response.raise_for_status()

        if not config.response_type:
            return response.json()
        else:
            return response.text


class SurveyGizmo(object):
    """
    """
    def __init__(self, **kwargs):
        self.config = _Config(self, **kwargs)
        self.api = _API(self)
        self.api._check_version()





def list(*args, **kwargs):
    """ Get list of all surveys.

        Optional params:
        - page:             page number
        - resultsperpage:   number of results per page
    """
    return "survey/", {}


def get(survey_id, *args, **kwargs):
    """ Get survey by id.

        Required params:
        - survey_id:    survey ID

        Optional params:
        - metaonly:     exclude page info
    """
    tail = "survey/%s" % survey_id
    params = {

    }
    params.update(kwargs)
    return tail, params


def create(title, type, *args, **kwargs):
    """ Create new survey object.

        Required params:
            - title   survey title
            - type    select from [survey, form, poll, quiz]

        Optional params:
            - status:                   select from [launched, closed, deleted]
            - theme:                    theme ID
            - team:                     team ID
            - options[internal_title]:  internal title
            - blockby:                  select from [NONE, IP, COOKIE]
    """
    tail = "survey/"
    params = {
        '_method': 'PUT',
    }
    params.update(kwargs)
    return tail, params


def change(survey_id, *args, **kwargs):
    """ Change existing survey object.

        Required params:
        - survey_id: survey ID

        Optional params:
        - title:                    survey title
        - status:                   select from [launched, closed, deleted]
        - theme:                    theme ID
        - team:                     team ID
        - options[internal_title]:  internal title
        - blockby:                  select from [NONE, IP, COOKIE]
    """
    tail = "survey/%s" % survey_id
    params = {
        '_method': 'POST',
    }
    params.update(kwargs)
    return tail, params


def copy(survey_id, title, *args, **kwargs):
    """ Copy new survey object from existing survey.

        Required params:
        - survey_id:    survey ID
        - title:        survey title

        Optional params:
        - status:                   select from [launched, closed, deleted]
        - theme:                    theme ID
        - team:                     team ID
        - options[internal_title]:  internal title
        - blockby:                  select from [NONE, IP, COOKIE]
    """
    tail = "survey/%s" % survey_id
    params = {
        '_method': 'POST',
        'title': title,
        'copy': 'true',
    }
    params.update(kwargs)
    return tail, params


def delete(survey_id, *args, **kwargs):
    """ Delete survey object.

        Required params:
        - survey_id: survey ID
    """
    tail = "survey/%s" % survey_id
    params = {
        '_method': 'DELETE',
    }
    params.update(kwargs)
    return tail, params




import rauth


class SGAuthService(rauth.OAuth1Service):
    """ This is a helper class that wraps `rauth.OAuth1Service` and slightly
        simplifies the process of authenciating with SurveyGizmo.
        For further details on the underlying implementation and inherited
        properties, refer to the rauth documentation.

        This class should primarily be used for retrieving an access_token,
        which can then be provided to the API. To do this, provide the
        `consumer_key` and `consumer_secret` given to you during application
        registration, and make the subsequent calls to `get_authorize_url` and
        `get_access_token`.

        `get_authorize_url` will generate a request token and provide a url
        that directs a user to authenticate your application with SurveyGizmo.
        SurveyGizmo will then make a request containing the `oauth_verifier`
        to the callback_url specified during registration.

        `get_access_token` takes the `oauth_verifier` provided by SurveyGizmo
        and trades it and your request token for an access token pair. This
        contains the `access_token` and `access_token_secret`. The token pair
        should be saved securely and then provided to the API for requests.


        This helper class can also be used to directly make custom calls to
        SurveyGizmo's REST API. After calling `get_access_token`, the service
        is able to create a session object through `get_session`, which will
        make the actual requests. Refer to the rauth documentation for more
        information on `Session` objects.
        If the original `SGAuthService` object has been discarded,
        reinstantiate it with the `access_token` and `access_token_secret`.

        :param consumer_key: Client consumer key, required for signing.
        :type consumer_key: str
        :param consumer_secret: Client consumer secret, required for signing.
        :type consumer_secret: str
        :param access_token: Client oauth token, required for authenticating.
        :type access_token: str
        :param access_token_secret: Client oauth token secret, required for authenticating.
        :type access_token_secret: str
    """
    def __init__(self, consumer_key, consumer_secret,
                 access_token=None, access_token_secret=None):

        self.access_token = access_token
        self.access_token_secret = access_token_secret
        super(SGAuthService, self).__init__(
            name="surveygizmo",
            consumer_key=consumer_key,
            consumer_secret=consumer_secret,
            request_token_url='https://restapi.surveygizmo.com/head/oauth/request_token',
            access_token_url='https://restapi.surveygizmo.com/head/oauth/access_token',
            authorize_url='https://restapi.surveygizmo.com/head/oauth/authenticate',
            base_url='https://restapi.surveygizmo.com/'
        )

    def get_authorize_url(self):
        """ Generates a request token and generates a URL for authorizing your
            application.
        """
        self._request_token, self._request_token_secret = self.get_request_token()
        return super(SGAuthService, self).get_authorize_url(self._request_token)

    def get_access_token(self, oauth_verifier):
        """ Retrieves an access token pair form SurveyGizmo. This invalidates the
            `oauth_verifier`.
            :param oauth_verifier: Verification code tied to an authorization request.
            :type oauth_verifier: str
        """
        access_pair = super(SGAuthService, self).get_access_token(
            self._request_token,
            self._request_token_secret,
            data={'oauth_verifier': oauth_verifier}
        )
        self.access_token, self.access_token_secret = access_pair
        return access_pair

    def get_session(self):
        """ Gets an authenticated session object. This session can make requests
            to SurveyGizmo's REST API.
        """
        token = (self.access_token, self.access_token_secret)
        return super(SGAuthService, self).get_session(token)





def list(*args, **kwargs):
    """ Get list of all account users.

        Optional params:
        - page:             page number
        - resultsperpage:   number of results per page
    """
    return "accountuser/", {}


def get(accountuser_id, *args, **kwargs):
    """ Get account user by id.

        Required params:
        - accountuser_id:    account user ID
    """
    tail = "accountuser/%s" % accountuser_id
    params = {

    }
    params.update(kwargs)
    return tail, params


def create(title, type, *args, **kwargs):
    """ Create new account user object.

        Required params:
            - email   email address

        Optional params:
            - username:
            - password:
            - team:                     team ID
            - create_access_token:
    """
    tail = "accountuser/"
    params = {
        '_method': 'PUT',
    }
    params.update(kwargs)
    return tail, params


def change(accountuser_id, *args, **kwargs):
    """ Change existing account user object.

        Required params:
        - accountuser_id: account user ID

        Optional params:
        - title:                    account user title
        - status:                   select from [launched, closed, deleted]
        - theme:                    theme ID
        - team:                     team ID
        - options[internal_title]:  internal title
        - blockby:                  select from [NONE, IP, COOKIE]
    """
    tail = "accountuser/%s" % accountuser_id
    params = {
        '_method': 'POST',
    }
    params.update(kwargs)
    return tail, params


def copy(accountuser_id, title, *args, **kwargs):
    """ Copy new account user object from existing account user.

        Required params:
        - accountuser_id:    account user ID
        - title:        account user title

        Optional params:
        - status:                   select from [launched, closed, deleted]
        - theme:                    theme ID
        - team:                     team ID
        - options[internal_title]:  internal title
        - blockby:                  select from [NONE, IP, COOKIE]
    """
    tail = "accountuser/%s" % accountuser_id
    params = {
        '_method': 'POST',
        'title': title,
        'copy': 'true',
    }
    params.update(kwargs)
    return tail, params


def delete(accountuser_id, *args, **kwargs):
    """ Delete account user object.

        Required params:
        - accountuser_id: accountuser ID
    """
    tail = "accountuser/%s" % accountuser_id
    params = {
        '_method': 'DELETE',
    }
    params.update(kwargs)
    return tail, params





def list(*args, **kwargs):
    """ Get list of all accounts.
    """
    return "account/", {}


def get(account_id, *args, **kwargs):
    """ Get account by id.

        Required params:
        - account_id:    survey ID
    """
    tail = "account/%s" % account_id
    params = {

    }
    params.update(kwargs)
    return tail, params



'''
    SurveyGizmo
    -----------

    A Python Wrapper for SurveyGizmo's restful API service

        >>> from surveygizmo import SurveyGizmo
        >>> sg = SurveyGizmo('v3')
        >>> sg.config.auth_method = 'user:md5'
        >>> sg.config.username = ...
        >>> sg.config.md5_hash = ...

        ...

        >>> surveys = sg.api.survey.list()
        >>> print surveys

'''

__title__ = 'surveygizmo'
__version_info__ = ('0', '0', '3')
__version__ = '.'.join(__version_info__)
__author__ = 'Ryan P Kilby'
__license__ = 'MIT'
__copyright__ = 'Copyright 2013 Ryan P Kilby'
__all__ = ['SurveyGizmo', 'ImproperlyConfigured', 'SGAuthService']


from .oauth_helper import SGAuthService
from .surveygizmo import SurveyGizmo, ImproperlyConfigured



import os
from setuptools import setup, find_packages
from surveygizmo import __version__

README = open(os.path.join(os.path.dirname(__file__), 'README.md')).read()

os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

setup(name='SurveyGizmo',
    version=__version__,
    description='A Python Wrapper for SurveyGizmo\'s restful API service.',
    long_description=README,
    author='Ryan P Kilby',
    author_email='rpkilby@ncsu.edu',
    keywords="Survey Gizmo SurveyGizmo surveygizmo",
    url='https://github.com/rpkilby/SurveyGizmo/',
    packages=find_packages(),
    install_requires=['requests==1.2.3', 'rauth'],
    license='BSD License',
    classifiers=(
        'Development Status :: 2 - Pre-Alpha',
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.7',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ),
)
